@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +17,7 @@
 
 """Base class for batch anomaly scores' REST calls
 
-   https://bigml.com/developers/batchanomalyscores
+   https://bigml.com/api/batchanomalyscores
 
 """
 
@@ -30,7 +30,7 @@ from bigml.bigmlconnection import DOWNLOAD_DIR
 from bigml.resourcehandler import ResourceHandler
 from bigml.resourcehandler import (check_resource_type,
                                    get_batch_anomaly_score_id)
-from bigml.resourcehandler import BATCH_ANOMALY_SCORE_PATH, ANOMALY_PATH
+from bigml.constants import BATCH_ANOMALY_SCORE_PATH, ANOMALY_PATH
 
 
 class BatchAnomalyScoreHandler(ResourceHandler):
@@ -46,7 +46,8 @@ class BatchAnomalyScoreHandler(ResourceHandler):
            instantiated independently.
 
         """
-        self.batch_anomaly_score_url = self.url + BATCH_ANOMALY_SCORE_PATH
+        self.batch_anomaly_score_url = self.prediction_base_url + \
+            BATCH_ANOMALY_SCORE_PATH
 
     def create_batch_anomaly_score(self, anomaly, dataset,
                                    args=None, wait_time=3, retries=10):
@@ -66,7 +67,7 @@ class BatchAnomalyScoreHandler(ResourceHandler):
             body = json.dumps(create_args)
             return self._create(self.batch_anomaly_score_url, body)
 
-    def get_batch_anomaly_score(self, batch_anomaly_score):
+    def get_batch_anomaly_score(self, batch_anomaly_score, query_string=''):
         """Retrieves a batch anomaly score.
 
            The batch_anomaly_score parameter should be a string containing the
@@ -82,7 +83,8 @@ class BatchAnomalyScoreHandler(ResourceHandler):
         batch_anomaly_score_id = get_batch_anomaly_score_id(
             batch_anomaly_score)
         if batch_anomaly_score_id:
-            return self._get("%s%s" % (self.url, batch_anomaly_score_id))
+            return self._get("%s%s" % (self.url, batch_anomaly_score_id),
+                             query_string=query_string)
 
     def download_batch_anomaly_score(self, batch_anomaly_score, filename=None):
         """Retrieves the batch anomaly score file.
