@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -25,9 +25,10 @@ in the cluster.
 import math
 import sys
 
-
+INDENT = " " * 4
 STATISTIC_MEASURES = [
-    'Minimum', 'Mean', 'Median', 'Maximum', 'Standard deviation']
+    'Minimum', 'Mean', 'Median', 'Maximum', 'Standard deviation', 'Sum',
+    'Sum squares', 'Variance']
 
 
 def cosine_distance2(terms, centroid_terms, scale):
@@ -70,11 +71,11 @@ class Centroid(object):
         for field_id, value in self.center.items():
             if isinstance(value, list):
                 # text field
-                terms = ([] if not field_id in term_sets else
+                terms = ([] if field_id not in term_sets else
                          term_sets[field_id])
                 distance2 += cosine_distance2(terms, value, scales[field_id])
             elif isinstance(value, basestring):
-                if not field_id in input_data or input_data[field_id] != value:
+                if field_id not in input_data or input_data[field_id] != value:
                     distance2 += 1 * scales[field_id] ** 2
             else:
                 distance2 += ((input_data[field_id] - value) *
@@ -88,9 +89,10 @@ class Centroid(object):
            centroid
 
         """
-        out.write(u"%s:\n" % self.name)
-        literal = u"    %s: %s\n"
+        out.write(u"%s%s:\n" % (INDENT, self.name))
+        literal = u"%s%s: %s\n"
         for measure_title in STATISTIC_MEASURES:
             measure = measure_title.lower().replace(" ", "_")
-            out.write(literal % (measure_title, self.distance[measure]))
+            out.write(literal % (INDENT * 2, measure_title,
+                                 self.distance[measure]))
         out.write("\n")
