@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +17,7 @@
 
 """Base class for evaluations' REST calls
 
-   https://bigml.com/developers/evaluations
+   https://bigml.com/api/evaluations
 
 """
 
@@ -30,7 +30,7 @@ except ImportError:
 from bigml.resourcehandler import ResourceHandler
 from bigml.resourcehandler import (check_resource_type,
                                    get_evaluation_id)
-from bigml.resourcehandler import EVALUATION_PATH, MODEL_PATH, ENSEMBLE_PATH
+from bigml.constants import SUPERVISED_PATHS, TIME_SERIES_PATH, EVALUATION_PATH
 
 
 class EvaluationHandler(ResourceHandler):
@@ -57,7 +57,9 @@ class EvaluationHandler(ResourceHandler):
         if args is not None:
             create_args.update(args)
 
-        model_types = [ENSEMBLE_PATH, MODEL_PATH]
+        model_types = SUPERVISED_PATHS[:]
+        model_types.append(TIME_SERIES_PATH)
+
         origin_resources_checked = self.check_origins(
             dataset, model, create_args, model_types=model_types,
             wait_time=wait_time, retries=retries)
@@ -66,7 +68,7 @@ class EvaluationHandler(ResourceHandler):
             body = json.dumps(create_args)
             return self._create(self.evaluation_url, body)
 
-    def get_evaluation(self, evaluation):
+    def get_evaluation(self, evaluation, query_string=''):
         """Retrieves an evaluation.
 
            The evaluation parameter should be a string containing the
@@ -80,7 +82,8 @@ class EvaluationHandler(ResourceHandler):
                             message="An evaluation id is needed.")
         evaluation_id = get_evaluation_id(evaluation)
         if evaluation_id:
-            return self._get("%s%s" % (self.url, evaluation_id))
+            return self._get("%s%s" % (self.url, evaluation_id),
+                             query_string=query_string)
 
     def list_evaluations(self, query_string=''):
         """Lists all your evaluations.
