@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2012-2015 BigML
+# Copyright 2012-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -18,19 +18,29 @@
 
 """
 import logging
-LOGGER = logging.getLogger('BigML')
-
 import numbers
 import math
 
+
+from bigml.util import PRECISION
+
+
+LOGGER = logging.getLogger('BigML')
+
 PLURALITY = 'plurality'
 CONFIDENCE = 'confidence weighted'
 PROBABILITY = 'probability weighted'
 THRESHOLD = 'threshold'
+BOOSTING = 'boosting'
 PLURALITY_CODE = 0
 CONFIDENCE_CODE = 1
 PROBABILITY_CODE = 2
 THRESHOLD_CODE = 3
+# negative combiner codes are meant for internal use only
+BOOSTING_CODE = -1
+# note that -2 and -3 codes are also used in BigMLer
+# COMBINATION = -2
+# AGGREGATION = -3
 
 PREDICTION_HEADERS = ['prediction', 'confidence', 'order', 'distribution',
                       'count']
@@ -38,19 +48,51 @@ COMBINATION_WEIGHTS = {
     PLURALITY: None,
     CONFIDENCE: 'confidence',
     PROBABILITY: 'probability',
-    THRESHOLD: None}
+    THRESHOLD: None,
+    BOOSTING: 'weight'}
 COMBINER_MAP = {
     PLURALITY_CODE: PLURALITY,
     CONFIDENCE_CODE: CONFIDENCE,
     PROBABILITY_CODE: PROBABILITY,
-    THRESHOLD_CODE: THRESHOLD}
+    THRESHOLD_CODE: THRESHOLD,
+    BOOSTING_CODE: BOOSTING}
 WEIGHT_KEYS = {
     PLURALITY: None,
     CONFIDENCE: ['confidence'],
     PROBABILITY: ['distribution', 'count'],
-    THRESHOLD: None}
+    THRESHOLD: None,
+    BOOSTING: ['weight']}
+BOOSTING_CLASS = 'class'
+CONFIDENCE_W = COMBINATION_WEIGHTS[CONFIDENCE]
 
 DEFAULT_METHOD = 0
+BINS_LIMIT = 32
+
+def weighted_sum(predictions, weight=None):
+    """Returns a weighted sum of the predictions
+
+    """
+    return sum([prediction["prediction"] * prediction[weight] for
+                prediction in predictions])
+
+
+def softmax(predictions):
+    """Returns the softmax values from a distribution given as a dictionary
+    like:
+        {"category": {"probability": probability, "order": order}}
+
+    """
+    total = 0.0
+    normalized = {}
+    for category, cat_info in predictions.items():
+        normalized[category] = { \
+            "probability": math.exp(cat_info["probability"]),
+            "order": cat_info["order"]}
+        total += normalized[category]["probability"]
+    return float('nan') if total == 0 else \
+        {category: {"probability": cat_info["probability"] / total,
+                    "order": cat_info["order"]}
+         for category, cat_info in normalized.items()}
 
 
 def ws_confidence(prediction, distribution, ws_z=1.96, ws_n=None):
@@ -74,7 +116,7 @@ def ws_confidence(prediction, distribution, ws_z=1.96, ws_n=None):
     if ws_p < 0:
         raise ValueError("The distribution weight must be a positive value")
     ws_norm = float(sum(distribution.values()))
-    if not ws_norm == 1.0:
+    if ws_norm != 1.0:
         ws_p = ws_p / ws_norm
     if ws_n is None:
         ws_n = ws_norm
@@ -87,7 +129,44 @@ def ws_confidence(prediction, distribution, ws_z=1.96, ws_n=None):
     ws_z2 = ws_z * ws_z
     ws_factor = ws_z2 / ws_n
     ws_sqrt = math.sqrt((ws_p * (1 - ws_p) + ws_factor / 4) / ws_n)
-    return (ws_p + ws_factor / 2 - ws_z * ws_sqrt) / (1 + ws_factor)
+    return round((ws_p + ws_factor / 2 - ws_z * ws_sqrt) / (1 + ws_factor),
+                 PRECISION)
+
+
+def merge_distributions(distribution, new_distribution):
+    """Adds up a new distribution structure to a map formatted distribution
+
+    """
+    for value, instances in new_distribution.items():
+        if value not in distribution:
+            distribution[value] = 0
+        distribution[value] += instances
+    return distribution
+
+
+def merge_bins(distribution, limit):
+    """Merges the bins of a regression distribution to the given limit number
+
+    """
+    length = len(distribution)
+    if limit < 1 or length <= limit or length < 2:
+        return distribution
+    index_to_merge = 2
+    shortest = float('inf')
+    for index in range(1, length):
+        distance = distribution[index][0] - distribution[index - 1][0]
+        if distance < shortest:
+            shortest = distance
+            index_to_merge = index
+    new_distribution = distribution[: index_to_merge - 1]
+    left = distribution[index_to_merge - 1]
+    right = distribution[index_to_merge]
+    new_bin = [(left[0] * left[1] + right[0] * right[1]) /
+               (left[1] + right[1]), left[1] + right[1]]
+    new_distribution.append(new_bin)
+    if index_to_merge < (length - 1):
+        new_distribution.extend(distribution[(index_to_merge + 1):])
+    return merge_bins(new_distribution, limit)
 
 
 class MultiVote(object):
@@ -96,83 +175,165 @@ class MultiVote(object):
     Uses a number of predictions to generate a combined prediction.
 
     """
+    @classmethod
+    def grouped_distribution(cls, instance):
+        """Returns a distribution formed by grouping the distributions of
+           each predicted node.
+
+        """
+        joined_distribution = {}
+        distribution_unit = 'counts'
+        for prediction in instance.predictions:
+            joined_distribution = merge_distributions(
+                joined_distribution,
+                dict((x[0], x[1]) for x in prediction['distribution']))
+            # when there's more instances, sort elements by their mean
+            distribution = [list(element) for element in
+                            sorted(joined_distribution.items(),
+                                   key=lambda x: x[0])]
+            if distribution_unit == 'counts':
+                distribution_unit = ('bins' if len(distribution) > BINS_LIMIT
+                                     else 'counts')
+            if distribution_unit != 'categories':
+                distribution = merge_bins(distribution, BINS_LIMIT)
+        return {'distribution': distribution,
+                'distribution_unit': distribution_unit}
 
     @classmethod
-    def avg(cls, instance, with_confidence=False):
+    def avg(cls, instance, full=False):
         """Returns the average of a list of numeric values.
 
-           If with_confidence is True, the combined confidence (as the
+           If full is True, the combined confidence (as the
            average of confidences of the multivote predictions) is also
            returned
         """
-        if (instance.predictions and with_confidence and
-                not all(['confidence' in prediction
+        if (instance.predictions and full and
+                not all([CONFIDENCE_W in prediction
                          for prediction in instance.predictions])):
             raise Exception("Not enough data to use the selected "
                             "prediction method. Try creating your"
                             " model anew.")
         total = len(instance.predictions)
         result = 0.0
+        median_result = 0.0
         confidence = 0.0
+        instances = 0
+        missing_confidence = 0
+        d_min = float('Inf')
+        d_max = float('-Inf')
         for prediction in instance.predictions:
             result += prediction['prediction']
-            if with_confidence:
-                confidence += prediction['confidence']
-        if with_confidence:
-            return ((result / total, confidence / total) if total > 0 else
-                    (float('nan'), 0))
+            if full:
+                if 'median' in prediction:
+                    median_result += prediction['median']
+                # some buggy models don't produce a valid confidence value
+                if prediction[CONFIDENCE_W] is not None and \
+                   prediction[CONFIDENCE_W] > 0:
+                    confidence += prediction[CONFIDENCE_W]
+                else:
+                    missing_confidence += 1
+                instances += prediction['count']
+                if 'min' in prediction and prediction['min'] < d_min:
+                    d_min = prediction['min']
+                if 'max' in prediction and prediction['max'] > d_max:
+                    d_max = prediction['max']
+        if full:
+            output = {'prediction': result / total if total > 0 else \
+                float('nan')}
+            # some strange models have no confidence
+            output.update(
+                {'confidence': round( \
+                    confidence / (total - missing_confidence), PRECISION) \
+                 if total > 0 else 0})
+            output.update(cls.grouped_distribution(instance))
+            output.update({'count': instances})
+            if median_result > 0:
+                output.update({
+                    'median': median_result / total if \
+                    total > 0 else float('nan')})
+            if d_min < float('Inf'):
+                output.update({'min': d_min})
+            if d_max > float('-Inf'):
+                output.update({'max': d_max})
+            return output
         return result / total if total > 0 else float('nan')
 
     @classmethod
-    def error_weighted(cls, instance, with_confidence=False):
+    def error_weighted(cls, instance, full=False):
         """Returns the prediction combining votes using error to compute weight
 
-           If with_confidences is true, the combined confidence (as the
+           If full is true, the combined confidence (as the
            error weighted average of the confidences of the multivote
            predictions) is also returned
         """
-        if (instance.predictions and with_confidence and
-                not all(['confidence' in prediction
+        if (instance.predictions and full and
+                not all([CONFIDENCE_W in prediction
                          for prediction in instance.predictions])):
             raise Exception("Not enough data to use the selected "
                             "prediction method. Try creating your"
                             " model anew.")
         top_range = 10
         result = 0.0
+        median_result = 0.0
+        instances = 0
+        d_min = float('Inf')
+        d_max = float('-Inf')
         normalization_factor = cls.normalize_error(instance, top_range)
         if normalization_factor == 0:
-            if with_confidence:
-                return float('nan'), 0
+            if full:
+                return {"prediction": float('nan')}
             else:
                 return float('nan')
-        if with_confidence:
+        if full:
             combined_error = 0.0
         for prediction in instance.predictions:
             result += prediction['prediction'] * prediction['_error_weight']
-            if with_confidence:
-                combined_error += (prediction['confidence'] *
-                                   prediction['_error_weight'])
+            if full:
+                if 'median' in prediction:
+                    median_result += (prediction['median'] *
+                                      prediction['_error_weight'])
+                instances += prediction['count']
+                if 'min' in prediction and prediction['min'] < d_min:
+                    d_min = prediction['min']
+                if 'max' in prediction and prediction['max'] > d_max:
+                    d_max = prediction['max']
+                # some buggy models don't produce a valid confidence value
+                if prediction[CONFIDENCE_W] is not None:
+                    combined_error += (prediction[CONFIDENCE_W] *
+                                       prediction['_error_weight'])
             del prediction['_error_weight']
-        if with_confidence:
-            return (result / normalization_factor,
-                    combined_error / normalization_factor)
-        else:
-            return result / normalization_factor
+        if full:
+            output = {'prediction': result / normalization_factor}
+            output.update({'confidence':
+                           round(combined_error / normalization_factor,
+                                 PRECISION)})
+            output.update(cls.grouped_distribution(instance))
+            output.update({'count': instances})
+            if median_result > 0:
+                output.update({'median': median_result / normalization_factor})
+            if d_min < float('Inf'):
+                output.update({'min': d_min})
+            if d_max > float('-Inf'):
+                output.update({'max': d_max})
+            return output
+        return result / normalization_factor
 
     @classmethod
     def normalize_error(cls, instance, top_range):
         """Normalizes error to a [0, top_range] and builds probabilities
 
         """
-        if instance.predictions and not all(['confidence' in prediction
+        if instance.predictions and not all([CONFIDENCE_W in prediction
                                              for prediction
                                              in instance.predictions]):
             raise Exception("Not enough data to use the selected "
                             "prediction method. Try creating your"
                             " model anew.")
 
-        error_values = [prediction['confidence']
-                        for prediction in instance.predictions]
+        error_values = []
+        for prediction in instance.predictions:
+            if prediction[CONFIDENCE_W] is not None:
+                error_values.append(prediction[CONFIDENCE_W])
         max_error = max(error_values)
         min_error = min(error_values)
         error_range = 1.0 * (max_error - min_error)
@@ -182,30 +343,36 @@ class MultiVote(object):
             # Then builds e^-[scaled error] and returns the normalization
             # factor to fit them between [0, 1]
             for prediction in instance.predictions:
-                delta = (min_error - prediction['confidence'])
+                delta = (min_error - prediction[CONFIDENCE_W])
                 prediction['_error_weight'] = math.exp(delta / error_range *
                                                        top_range)
                 normalize_factor += prediction['_error_weight']
         else:
             for prediction in instance.predictions:
                 prediction['_error_weight'] = 1
-            normalize_factor = len(instance.predictions)
+            normalize_factor = len(error_values)
         return normalize_factor
 
-    def __init__(self, predictions):
+    def __init__(self, predictions, boosting_offsets=None):
         """Init method, builds a MultiVote with a list of predictions
-
-           The constuctor expects a list of well formed predictions like:
-                {'prediction': 'Iris-setosa', 'confidence': 0.7}
-            Each prediction can also contain an 'order' key that is used
-            to break even in votations. The list order is used by default.
+        The constuctor expects a list of well formed predictions like:
+            {'prediction': 'Iris-setosa', 'confidence': 0.7}
+        Each prediction can also contain an 'order' key that is used
+        to break even in votations. The list order is used by default.
+        The boosting_offsets can contain the offset used in boosting models, so
+        whenever is not None votes will be considered from boosting models.
         """
         self.predictions = []
+        self.boosting = boosting_offsets is not None
+        self.boosting_offsets = boosting_offsets
+
         if isinstance(predictions, list):
             self.predictions.extend(predictions)
         else:
             self.predictions.append(predictions)
+
         if not all(['order' in prediction for prediction in predictions]):
+
             for i in range(len(self.predictions)):
                 self.predictions[i]['order'] = i
 
@@ -213,6 +380,9 @@ class MultiVote(object):
         """Returns True if all the predictions are numbers
 
         """
+        if self.boosting:
+            return any(prediction.get('class') is None for
+                       prediction in self.predictions)
         return all([isinstance(prediction['prediction'], numbers.Number)
                     for prediction in self.predictions])
 
@@ -228,14 +398,13 @@ class MultiVote(object):
             return self.predictions[-1]['order'] + 1
         return 0
 
-    def combine(self, method=DEFAULT_METHOD, with_confidence=False,
-                options=None):
+    def combine(self, method=DEFAULT_METHOD, options=None, full=False):
         """Reduces a number of predictions voting for classification and
            averaging predictions for regression.
 
            method will determine the voting method (plurality, confidence
            weighted, probability weighted or threshold).
-           If with_confidence is true, the combined confidence (as a weighted
+           If full is true, the combined confidence (as a weighted
            average of the confidences of votes for the combined prediction)
            will also be given.
         """
@@ -246,20 +415,32 @@ class MultiVote(object):
         method = COMBINER_MAP.get(method, COMBINER_MAP[DEFAULT_METHOD])
         keys = WEIGHT_KEYS.get(method, None)
         # and all predictions should have the weight-related keys
-        if not keys is None:
+        if keys is not None:
             for key in keys:
                 if not all([key in prediction for prediction
                             in self.predictions]):
                     raise Exception("Not enough data to use the selected "
                                     "prediction method. Try creating your"
                                     " model anew.")
-        if self.is_regression():
+        if self.boosting:
             for prediction in self.predictions:
-                if prediction['confidence'] is None:
-                    prediction['confidence'] = 0
+                if prediction[COMBINATION_WEIGHTS[BOOSTING]] is None:
+                    prediction[COMBINATION_WEIGHTS[BOOSTING]] = 0
+            if self.is_regression():
+                # sum all gradients weighted by their "weight" plus the
+                # boosting offset
+                return weighted_sum(self.predictions, weight="weight") + \
+                    self.boosting_offsets
+            else:
+                return self.classification_boosting_combiner( \
+                    options, full=full)
+        elif self.is_regression():
+            for prediction in self.predictions:
+                if prediction[CONFIDENCE_W] is None:
+                    prediction[CONFIDENCE_W] = 0
             function = NUMERICAL_COMBINATION_METHODS.get(method,
                                                          self.__class__.avg)
-            return function(self, with_confidence=with_confidence)
+            return function(self, full=full)
         else:
             if method == THRESHOLD:
                 if options is None:
@@ -272,7 +453,7 @@ class MultiVote(object):
                 predictions = self
             return predictions.combine_categorical(
                 COMBINATION_WEIGHTS.get(method, None),
-                with_confidence=with_confidence)
+                full=full)
 
     def probability_weight(self):
         """Reorganizes predictions depending on training data probability
@@ -280,7 +461,7 @@ class MultiVote(object):
         """
         predictions = []
         for prediction in self.predictions:
-            if not 'distribution' in prediction or not 'count' in prediction:
+            if 'distribution' not in prediction or 'count' not in prediction:
                 raise Exception("Probability weighting is not available "
                                 "because distribution information is missing.")
             total = prediction['count']
@@ -290,10 +471,11 @@ class MultiVote(object):
                                 "as number of instances in a node" % total)
             order = prediction['order']
             for prediction, instances in prediction['distribution']:
-                predictions.append({'prediction': prediction,
-                                    'probability': float(instances) / total,
-                                    'count': instances,
-                                    'order': order})
+                predictions.append({ \
+                    'prediction': prediction,
+                    'probability': round(float(instances) / total, PRECISION),
+                    'count': instances,
+                    'order': order})
         return predictions
 
     def combine_distribution(self, weight_label='probability'):
@@ -311,7 +493,7 @@ class MultiVote(object):
         distribution = {}
         total = 0
         for prediction in self.predictions:
-            if not prediction['prediction'] in distribution:
+            if prediction['prediction'] not in distribution:
                 distribution[prediction['prediction']] = 0.0
             distribution[prediction['prediction']] += prediction[weight_label]
             total += prediction['count']
@@ -322,7 +504,7 @@ class MultiVote(object):
             distribution = []
         return distribution, total
 
-    def combine_categorical(self, weight_label=None, with_confidence=False):
+    def combine_categorical(self, weight_label=None, full=False):
         """Returns the prediction combining votes by using the given weight:
 
             weight_label can be set as:
@@ -330,57 +512,71 @@ class MultiVote(object):
             'confidence':  confidence weighted (confidence as a vote value)
             'probability': probability weighted (probability as a vote value)
 
-            If with_confidence is true, the combined confidence (as a weighted
+            If full is true, the combined confidence (as a weighted
             average of the confidences of the votes for the combined
             prediction) will also be given.
         """
         mode = {}
+        instances = 0
         if weight_label is None:
             weight = 1
         for prediction in self.predictions:
-            if not weight_label is None:
-                if not weight_label in COMBINATION_WEIGHTS.values():
+            if weight_label is not None:
+                if weight_label not in COMBINATION_WEIGHTS.values():
                     raise Exception("Wrong weight_label value.")
-                if not weight_label in prediction:
+                if weight_label not in prediction:
                     raise Exception("Not enough data to use the selected "
                                     "prediction method. Try creating your"
                                     " model anew.")
                 else:
                     weight = prediction[weight_label]
             category = prediction['prediction']
+            if full:
+                instances += prediction['count']
             if category in mode:
                 mode[category] = {"count": mode[category]["count"] + weight,
                                   "order": mode[category]["order"]}
             else:
                 mode[category] = {"count": weight,
                                   "order": prediction['order']}
-
         prediction = sorted(mode.items(), key=lambda x: (x[1]['count'],
-                                                         -x[1]['order']),
+                                                         -x[1]['order'],
+                                                         x[0]),
                             reverse=True)[0][0]
-
-        if with_confidence:
+        if full:
+            output = {'prediction': prediction}
             if 'confidence' in self.predictions[0]:
-                return self.weighted_confidence(prediction,
-                                                weight_label)
+                prediction, combined_confidence = self.weighted_confidence(
+                    prediction, weight_label)
             # if prediction had no confidence, compute it from distribution
             else:
-                combined_distribution = self.combine_distribution()
-                distribution, count = combined_distribution
-                combined_confidence = ws_confidence(prediction, distribution,
-                                                    ws_n=count)
-                return prediction, combined_confidence
+                if 'probability' in self.predictions[0]:
+                    combined_distribution = self.combine_distribution()
+                    distribution, count = combined_distribution
+                    combined_confidence = ws_confidence(prediction,
+                                                        distribution,
+                                                        ws_n=count)
+            output.update({'confidence':
+                           round(combined_confidence, PRECISION)})
+            if 'probability' in self.predictions[0]:
+                for prediction in self.predictions:
+                    if prediction['prediction'] == output['prediction']:
+                        output['probability'] = prediction['probability']
+            if 'distribution' in self.predictions[0]:
+                output.update(self.__class__.grouped_distribution(self))
+            output.update({'count': instances})
+            return output
         return prediction
 
     def weighted_confidence(self, combined_prediction, weight_label):
         """Compute the combined weighted confidence from a list of predictions
 
         """
-        predictions = filter(lambda x: x['prediction'] == combined_prediction,
-                             self.predictions)
+        predictions = [prediction for prediction in self.predictions \
+            if prediction['prediction'] == combined_prediction]
         if (weight_label is not None and
                 (not isinstance(weight_label, basestring) or
-                 any([not 'confidence' or not weight_label in prediction
+                 any([not CONFIDENCE_W or weight_label not in prediction
                       for prediction in predictions]))):
             raise ValueError("Not enough data to use the selected "
                              "prediction method. Lacks %s information." %
@@ -391,12 +587,48 @@ class MultiVote(object):
         for prediction in predictions:
             if weight_label is not None:
                 weight = prediction[weight_label]
-            final_confidence += weight * prediction['confidence']
+            final_confidence += weight * prediction[CONFIDENCE_W]
             total_weight += weight
         final_confidence = (final_confidence / total_weight
                             if total_weight > 0 else float('nan'))
         return combined_prediction, final_confidence
 
+    def classification_boosting_combiner(self, options, full=False):
+        """Combines the predictions for a boosted classification ensemble
+        Applies the regression boosting combiner, but per class. Tie breaks
+        use the order of the categories in the ensemble summary to decide.
+
+        """
+        grouped_predictions = {}
+        for prediction in self.predictions:
+            if prediction.get(BOOSTING_CLASS) is not None:
+                objective_class = prediction.get(BOOSTING_CLASS)
+                if grouped_predictions.get(objective_class) is None:
+                    grouped_predictions[objective_class] = []
+                grouped_predictions[objective_class].append(prediction)
+        categories = options.get("categories", [])
+        predictions = {key: { \
+            "probability": weighted_sum(value, weight="weight") + \
+                self.boosting_offsets.get(key, 0),
+            "order": categories.index(key)} for
+                       key, value in grouped_predictions.items()}
+        predictions = softmax(predictions)
+        predictions = sorted( \
+            predictions.items(), key=lambda(x): \
+            (- x[1]["probability"], x[1]["order"]))
+        prediction, prediction_info = predictions[0]
+        confidence = round(prediction_info["probability"], PRECISION)
+        if full:
+            return {"prediction": prediction,
+                    "probability": confidence, \
+                "probabilities": [ \
+                    {"category": prediction,
+                     "probability": round(prediction_info["probability"],
+                                          PRECISION)}
+                    for prediction, prediction_info in predictions]}
+        else:
+            return prediction
+
     def append(self, prediction_info):
         """Adds a new prediction into a list of predictions
 
@@ -413,15 +645,21 @@ class MultiVote(object):
            - count: the total number of instances of the training set in the
                     node
         """
-        if (isinstance(prediction_info, dict) and
-                'prediction' in prediction_info):
-            order = self.next_order()
-            prediction_info['order'] = order
-            self.predictions.append(prediction_info)
-        else:
-            LOGGER.warning("Failed to add the prediction.\n"
-                           "The minimal key for the prediction is 'prediction'"
-                           ":\n{'prediction': 'Iris-virginica'")
+        if isinstance(prediction_info, dict):
+            if 'prediction' in prediction_info:
+                order = self.next_order()
+                prediction_info['order'] = order
+                self.predictions.append(prediction_info)
+            else:
+                LOGGER.warning("Failed to add the prediction.\n"
+                               "The minimal key for the prediction is "
+                               "'prediction': "
+                               "\n{'prediction': 'Iris-virginica'")
+        """
+        elif isinstance(prediction_info, list):
+            if self.probabilities:
+                self.predictions.append(prediction_info)
+        """
 
     def single_out_category(self, options):
         """Singles out the votes for a chosen category and returns a prediction
@@ -429,7 +667,7 @@ class MultiVote(object):
            threshold.
 
         """
-        if options is None or any(not option in options for option in
+        if options is None or any(option not in options for option in
                                   ["threshold", "category"]):
             raise Exception("No category and threshold information was"
                             " found. Add threshold and category info."
