@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +17,7 @@
 
 """Base class for predictions' REST calls
 
-   https://bigml.com/developers/predictions
+   https://bigml.com/api/predictions
 
 """
 
@@ -29,10 +29,10 @@ except ImportError:
 
 from bigml.resourcehandler import ResourceHandler
 from bigml.resourcehandler import (check_resource_type, get_prediction_id,
-                                   check_resource, get_ensemble_id,
-                                   get_model_id, get_resource_type)
-from bigml.resourcehandler import (PREDICTION_PATH, ENSEMBLE_PATH, MODEL_PATH,
-                                   TINY_RESOURCE, LOGGER)
+                                   check_resource, get_resource_id,
+                                   get_resource_type)
+from bigml.constants import SUPERVISED_PATHS, TINY_RESOURCE, PREDICTION_PATH
+
 
 class PredictionHandler(ResourceHandler):
     """This class is used by the BigML class as
@@ -47,37 +47,35 @@ class PredictionHandler(ResourceHandler):
            instantiated independently.
 
         """
-        self.prediction_url = self.prediction_url + PREDICTION_PATH
+        self.prediction_url = self.prediction_base_url + PREDICTION_PATH
 
     def create_prediction(self, model, input_data=None,
-                          args=None, wait_time=3, retries=10, by_name=True):
+                          args=None, wait_time=3, retries=10):
         """Creates a new prediction.
            The model parameter can be:
-            - a simple model
+            - a simple tree model
+            - a simple logistic regression model
             - an ensemble
-           The by_name argument is now deprecated. It will be removed.
+            - a deepnet
+            . a linear regression
+            - a fusion
+           Note that the old `by_name` argument has been deprecated.
 
         """
-        ensemble_id = None
         model_id = None
 
         resource_type = get_resource_type(model)
-        if resource_type == ENSEMBLE_PATH:
-            ensemble_id = get_ensemble_id(model)
-            if ensemble_id is not None:
-                check_resource(ensemble_id,
-                               query_string=TINY_RESOURCE,
-                               wait_time=wait_time, retries=retries,
-                               raise_on_error=True, api=self)
-        elif resource_type == MODEL_PATH:
-            model_id = get_model_id(model)
+        if resource_type not in SUPERVISED_PATHS:
+            raise Exception("A supervised model resource id is needed"
+                            " to create a prediction. %s found." %
+                            resource_type)
+
+        model_id = get_resource_id(model)
+        if model_id is not None:
             check_resource(model_id,
                            query_string=TINY_RESOURCE,
                            wait_time=wait_time, retries=retries,
                            raise_on_error=True, api=self)
-        else:
-            raise Exception("A model or ensemble id is needed to create a"
-                            " prediction. %s found." % resource_type)
 
         if input_data is None:
             input_data = {}
@@ -86,18 +84,15 @@ class PredictionHandler(ResourceHandler):
             create_args.update(args)
         create_args.update({
             "input_data": input_data})
-        if ensemble_id is None:
+        if model_id is not None:
             create_args.update({
                 "model": model_id})
-        else:
-            create_args.update({
-                "ensemble": ensemble_id})
 
         body = json.dumps(create_args)
         return self._create(self.prediction_url, body,
                             verify=self.verify_prediction)
 
-    def get_prediction(self, prediction):
+    def get_prediction(self, prediction, query_string=''):
         """Retrieves a prediction.
 
         """
@@ -105,7 +100,8 @@ class PredictionHandler(ResourceHandler):
                             message="A prediction id is needed.")
         prediction_id = get_prediction_id(prediction)
         if prediction_id:
-            return self._get("%s%s" % (self.url, prediction_id))
+            return self._get("%s%s" % (self.url, prediction_id),
+                             query_string=query_string)
 
     def list_predictions(self, query_string=''):
         """Lists all your predictions.
