@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2014-2015 BigML
+# Copyright 2014-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -17,7 +17,7 @@
 
 """Base class for projects' REST calls
 
-   https://bigml.com/developers/projects
+   https://bigml.com/api/projects
 
 """
 
@@ -28,9 +28,8 @@ except ImportError:
 
 
 from bigml.resourcehandler import ResourceHandler
-from bigml.resourcehandler import (check_resource_type, resource_is_ready,
-                                   get_project_id)
-from bigml.resourcehandler import PROJECT_PATH, LOGGER
+from bigml.resourcehandler import check_resource_type, get_project_id
+from bigml.constants import PROJECT_PATH
 
 
 class ProjectHandler(ResourceHandler):
@@ -48,16 +47,16 @@ class ProjectHandler(ResourceHandler):
         """
         self.project_url = self.url + PROJECT_PATH
 
-    def create_project(self, args=None, wait_time=3, retries=10):
+    def create_project(self, args=None):
         """Creates a project.
 
         """
         if args is None:
             args = {}
         body = json.dumps(args)
-        return self._create(self.project_url, body)
+        return self._create(self.project_url, body, organization=True)
 
-    def get_project(self, project):
+    def get_project(self, project, query_string=''):
         """Retrieves a project.
 
            The project parameter should be a string containing the
@@ -71,13 +70,14 @@ class ProjectHandler(ResourceHandler):
                             message="A project id is needed.")
         project_id = get_project_id(project)
         if project_id:
-            return self._get("%s%s" % (self.url, project_id))
+            return self._get("%s%s" % (self.url, project_id),
+                             query_string=query_string, organization=True)
 
     def list_projects(self, query_string=''):
         """Lists all your projects.
 
         """
-        return self._list(self.project_url, query_string)
+        return self._list(self.project_url, query_string, organization=True)
 
     def update_project(self, project, changes):
         """Updates a project.
@@ -88,7 +88,8 @@ class ProjectHandler(ResourceHandler):
         project_id = get_project_id(project)
         if project_id:
             body = json.dumps(changes)
-            return self._update("%s%s" % (self.url, project_id), body)
+            return self._update("%s%s" % (self.url, project_id), body,
+                                organization=True)
 
     def delete_project(self, project):
         """Deletes a project.
@@ -98,4 +99,5 @@ class ProjectHandler(ResourceHandler):
                             message="A project id is needed.")
         project_id = get_project_id(project)
         if project_id:
-            return self._delete("%s%s" % (self.url, project_id))
+            return self._delete("%s%s" % (self.url, project_id),
+                                organization=True)
