@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2013-2015 BigML
+# Copyright 2013-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -32,21 +32,15 @@ except ImportError:
     pass
 
 from bigml.predicate import Predicate
+from bigml.prediction import Prediction
 from bigml.predicate import TM_TOKENS, TM_FULL_TERM, TM_ALL
-from bigml.util import sort_fields, slugify, split, utf8
-from bigml.multivote import ws_confidence
-
-
-# Map operator str to its corresponding python operator
-PYTHON_OPERATOR = {
-    "<": "<",
-    "<=": "<=",
-    "=": "==",
-    "!=": "!=",
-    "/=": "!=",
-    ">=": ">=",
-    ">": ">"
-}
+from bigml.util import sort_fields, slugify, split, utf8, PRECISION
+from bigml.multivote import ws_confidence, merge_distributions, merge_bins
+from bigml.multivote import BINS_LIMIT
+from bigml.tree_utils import tableau_string, filter_nodes, missing_branch, \
+    none_value, one_branch
+from bigml.tree_utils import PYTHON_OPERATOR, MAX_ARGS_LENGTH, INDENT, \
+    TERM_OPTIONS, ITEM_OPTIONS
 
 MISSING_OPERATOR = {
     "=": "is",
@@ -58,16 +52,10 @@ T_MISSING_OPERATOR = {
     "!=": "NOT ISNULL("
 }
 
-
-MAX_ARGS_LENGTH = 10
-
-INDENT = u'    '
-
-TERM_OPTIONS = ["case_sensitive", "token_mode"]
-
 LAST_PREDICTION = 0
 PROPORTIONAL = 1
-BINS_LIMIT = 32
+
+DISTRIBUTION_GROUPS = ['bins', 'counts', 'categories']
 
 
 def get_instances(distribution):
@@ -77,42 +65,6 @@ def get_instances(distribution):
     return sum(x[1] for x in distribution) if distribution else 0
 
 
-def merge_distributions(distribution, new_distribution):
-    """Adds up a new distribution structure to a map formatted distribution
-
-    """
-    for value, instances in new_distribution.items():
-        if not value in distribution:
-            distribution[value] = 0
-        distribution[value] += instances
-    return distribution
-
-
-def merge_bins(distribution, limit):
-    """Merges the bins of a regression distribution to the given limit number
-
-    """
-    length = len(distribution)
-    if limit < 1 or length <= limit or length < 2:
-        return distribution
-    index_to_merge = 2
-    shortest = float('inf')
-    for index in range(1, length):
-        distance = distribution[index][0] - distribution[index - 1][0]
-        if distance < shortest:
-            shortest = distance
-            index_to_merge = index
-    new_distribution = distribution[: index_to_merge - 1]
-    left = distribution[index_to_merge - 1]
-    right = distribution[index_to_merge]
-    new_bin = [(left[0] * left[1] + right[0] * right[1]) /
-               (left[1] + right[1]), left[1] + right[1]]
-    new_distribution.append(new_bin)
-    if index_to_merge < (length - 1):
-        new_distribution.extend(distribution[(index_to_merge + 1):])
-    return merge_bins(new_distribution, limit)
-
-
 def mean(distribution):
     """Computes the mean of a distribution in the [[point, instances]] syntax
 
@@ -159,56 +111,31 @@ def regression_error(distribution_variance, population, r_z=1.96):
     return float('nan')
 
 
-def tableau_string(text):
-    """Transforms to a string representation in Tableau
+def extract_distribution(summary):
+    """Extracts the distribution info from the objective_summary structure
+       in any of its grouping units: bins, counts or categories
 
     """
-    value = repr(text)
-    if isinstance(text, unicode):
-        return value[1:]
-    return value
+    for group in DISTRIBUTION_GROUPS:
+        if group in summary:
+            return group, summary.get(group)
 
 
-def filter_nodes(nodes_list, ids=None, subtree=True):
-    """Filters the contents of a nodes_list. If any of the nodes is in the
-       ids list, the rest of nodes are removed. If none is in the ids list
-       we include or exclude the nodes depending on the subtree flag.
+def dist_median(distribution, count):
+    """Returns the median value for a distribution
 
     """
-    if not nodes_list:
-        return None
-    nodes = nodes_list[:]
-    if ids is not None:
-        for node in nodes:
-            if node.id in ids:
-                nodes = [node]
-                return nodes
-    if not subtree:
-        nodes = []
-    return nodes
-
-
-def missing_branch(children):
-    """Checks if the missing values are assigned to a special branch
-
-    """
-    return any([child.predicate.missing for child in children])
-
-
-def none_value(children):
-    """Checks if the predicate has a None value
-
-    """
-    return any([child.predicate.value is None for child in children])
-
-
-def one_branch(children, input_data):
-    """Check if there's only one branch to be followed
-
-    """
-    missing = split(children) in input_data
-    return (missing or missing_branch(children)
-            or none_value(children))
+    counter = 0
+    previous_value = None
+    for value, instances in distribution:
+        counter += instances
+        if counter > count / 2.0:
+            if (not count % 2 and (counter - 1) == (count / 2) and
+                    previous_value is not None):
+                return (value + previous_value) / 2.0
+            return value
+        previous_value = value
+    return None
 
 
 class Tree(object):
@@ -217,7 +144,7 @@ class Tree(object):
     """
     def __init__(self, tree, fields, objective_field=None,
                  root_distribution=None, parent_id=None, ids_map=None,
-                 subtree=True):
+                 subtree=True, tree_info=None):
 
         self.fields = fields
         self.objective_id = objective_field
@@ -242,36 +169,54 @@ class Tree(object):
         children = []
         if 'children' in tree:
             for child in tree['children']:
-                children.append(Tree(child,
-                                     self.fields,
-                                     objective_field=objective_field,
-                                     parent_id=self.id,
-                                     ids_map=ids_map,
-                                     subtree=subtree))
+                children.append(self.__class__( \
+                    child,
+                    self.fields,
+                    objective_field=objective_field,
+                    parent_id=self.id,
+                    ids_map=ids_map,
+                    subtree=subtree,
+                    tree_info=tree_info))
 
         self.children = children
         self.regression = self.is_regression()
+        tree_info['regression'] = (self.regression and
+                                   tree_info.get('regression', True))
         self.count = tree['count']
         self.confidence = tree.get('confidence', None)
         self.distribution = None
+        self.max = None
+        self.min = None
+        self.weighted = False
+        summary = None
         if 'distribution' in tree:
             self.distribution = tree['distribution']
         elif 'objective_summary' in tree:
             summary = tree['objective_summary']
-            if 'bins' in summary:
-                self.distribution = summary['bins']
-            elif 'counts' in summary:
-                self.distribution = summary['counts']
-            elif 'categories' in summary:
-                self.distribution = summary['categories']
+            (self.distribution_unit,
+             self.distribution) = extract_distribution(summary)
+            if 'weighted_objective_summary' in tree:
+                summary = tree['weighted_objective_summary']
+                (self.weighted_distribution_unit,
+                 self.weighted_distribution) = extract_distribution(summary)
+                self.weight = tree['weight']
+                self.weighted = True
         else:
             summary = root_distribution
-            if 'bins' in summary:
-                self.distribution = summary['bins']
-            elif 'counts' in summary:
-                self.distribution = summary['counts']
-            elif 'categories' in summary:
-                self.distribution = summary['categories']
+            (self.distribution_unit,
+             self.distribution) = extract_distribution(summary)
+        if self.regression:
+            tree_info['max_bins'] = max(tree_info.get('max_bins', 0),
+                                        len(self.distribution))
+            self.median = None
+            if summary:
+                self.median = summary.get('median')
+            if not self.median:
+                self.median = dist_median(self.distribution, self.count)
+            self.max = summary.get('maximum') or \
+                max([value for [value, _] in self.distribution])
+            self.min = summary.get('minimum') or \
+                min([value for [value, _] in self.distribution])
         self.impurity = None
         if not self.regression and self.distribution is not None:
             self.impurity = self.gini_impurity()
@@ -284,9 +229,9 @@ class Tree(object):
         purity = 0.0
         if self.distribution is None:
             return None
-        for category, instances in self.distribution:
-           purity += math.pow(instances / float(self.count), 2)
-        return (1.0 - purity) / 2
+        for _, instances in self.distribution:
+            purity += math.pow(instances / float(self.count), 2)
+        return 1.0 - purity
 
     def list_fields(self, out):
         """Lists a description of the model's fields.
@@ -331,7 +276,7 @@ class Tree(object):
         if path is None:
             path = []
         if not isinstance(self.predicate, bool):
-            path.append(self.predicate.to_LISP_rule(self.fields))
+            path.append(self.predicate.to_lisp_rule(self.fields))
 
         if self.children:
             for child in self.children:
@@ -339,13 +284,17 @@ class Tree(object):
                                            filter_function=filter_function)
         else:
             leaf = {
-                    'id': self.id,
-                    'confidence': self.confidence,
-                    'count': self.count,
-                    'distribution': self.distribution,
-                    'impurity': self.impurity,
-                    'output': self.output,
-                    'path': path}
+                'id': self.id,
+                'confidence': self.confidence,
+                'count': self.count,
+                'distribution': self.distribution,
+                'impurity': self.impurity,
+                'output': self.output,
+                'path': path}
+            if hasattr(self, 'weighted_distribution'):
+                leaf.update( \
+                    {"weighted_distribution": self.weighted_distribution,
+                     "weight": self.weight})
             if (not hasattr(filter_function, '__call__')
                     or filter_function(leaf)):
                 leaves += [leaf]
@@ -368,7 +317,11 @@ class Tree(object):
             path = []
         if missing_strategy == PROPORTIONAL:
             (final_distribution,
-             last_node) = self.predict_proportional(input_data, path=path)
+             d_min,
+             d_max,
+             last_node,
+             population,
+             parent_node) = self.predict_proportional(input_data, path=path)
 
             if self.regression:
                 # singular case:
@@ -376,39 +329,102 @@ class Tree(object):
                 if len(final_distribution.items()) == 1:
                     prediction, instances = final_distribution.items()[0]
                     if instances == 1:
-                        return (last_node.output, path, last_node.confidence,
-                                last_node.distribution, instances)               
+                        return Prediction(
+                            last_node.output,
+                            path,
+                            last_node.confidence,
+                            distribution=(last_node.distribution if not  \
+                                self.weighted else \
+                                last_node.weighted_distribution),
+                            count=instances,
+                            median=last_node.median,
+                            distribution_unit=last_node.distribution_unit,
+                            children=last_node.children,
+                            d_min=last_node.min,
+                            d_max=last_node.max)
                 # when there's more instances, sort elements by their mean
                 distribution = [list(element) for element in
                                 sorted(final_distribution.items(),
                                        key=lambda x: x[0])]
+                distribution_unit = ('bins' if len(distribution) > BINS_LIMIT
+                                     else 'counts')
                 distribution = merge_bins(distribution, BINS_LIMIT)
-                prediction = mean(distribution)
                 total_instances = sum([instances
                                        for _, instances in distribution])
-                confidence = regression_error(
-                    unbiased_sample_variance(distribution, prediction),
-                    total_instances)
-                return (prediction, path, confidence,
-                        distribution, total_instances)
+                if len(distribution) == 1:
+                    # where there's only one bin, there will be no error, but
+                    # we use a correction derived from the parent's error
+                    prediction = distribution[0][0]
+                    if total_instances < 2:
+                        total_instances = 1
+                    try:
+                        # some strange models can have nodes with no confidence
+                        confidence = round(parent_node.confidence /
+                                           math.sqrt(total_instances),
+                                           PRECISION)
+                    except AttributeError:
+                        confidence = None
+                else:
+                    prediction = mean(distribution)
+                    # weighted trees use the unweighted population to
+                    # compute the associated error
+                    confidence = round(regression_error(
+                        unbiased_sample_variance(distribution, prediction),
+                        population), PRECISION)
+                return Prediction(
+                    prediction,
+                    path,
+                    confidence,
+                    distribution=distribution,
+                    count=total_instances,
+                    median=dist_median(distribution, total_instances),
+                    distribution_unit=distribution_unit,
+                    children=last_node.children,
+                    d_min=d_min,
+                    d_max=d_max)
             else:
                 distribution = [list(element) for element in
                                 sorted(final_distribution.items(),
                                        key=lambda x: (-x[1], x[0]))]
-                return (distribution[0][0], path,
-                        ws_confidence(distribution[0][0], final_distribution),
-                        distribution, get_instances(distribution))
+                return Prediction(
+                    distribution[0][0],
+                    path,
+                    ws_confidence(distribution[0][0], final_distribution,
+                                  ws_n=population),
+                    distribution=distribution,
+                    count=population,
+                    median=None,
+                    distribution_unit='categorical',
+                    children=last_node.children)
 
         else:
             if self.children:
                 for child in self.children:
                     if child.predicate.apply(input_data, self.fields):
                         path.append(child.predicate.to_rule(self.fields))
-                        return child.predict(input_data, path)
-            return (self.output, path, self.confidence,
-                    self.distribution, get_instances(self.distribution))
+                        return child.predict(input_data, path=path)
 
-    def predict_proportional(self, input_data, path=None, missing_found=False):
+            if self.weighted:
+                output_distribution = self.weighted_distribution
+                output_unit = self.weighted_distribution_unit
+            else:
+                output_distribution = self.distribution
+                output_unit = self.distribution_unit
+
+            return Prediction(
+                self.output,
+                path,
+                self.confidence,
+                distribution=output_distribution,
+                count=self.count,
+                median=None if not self.regression else self.median,
+                distribution_unit=output_unit,
+                children=self.children,
+                d_min=None if not self.regression else self.min,
+                d_max=None if not self.regression else self.max)
+
+    def predict_proportional(self, input_data, path=None,
+                             missing_found=False, median=False, parent=None):
         """Makes a prediction based on a number of field values averaging
            the predictions of the leaves that fall in a subtree.
 
@@ -424,26 +440,45 @@ class Tree(object):
 
         final_distribution = {}
         if not self.children:
+            distribution = self.distribution if not self.weighted else \
+                self.weighted_distribution
             return (merge_distributions({}, dict((x[0], x[1])
-                                                 for x in self.distribution)),
-                    self)
-        if one_branch(self.children, input_data):
+                                                 for x in distribution)),
+                    self.min, self.max, self, self.count, parent)
+        if one_branch(self.children, input_data) or \
+                self.fields[split(self.children)]["optype"] in \
+                ["text", "items"]:
             for child in self.children:
                 if child.predicate.apply(input_data, self.fields):
                     new_rule = child.predicate.to_rule(self.fields)
-                    if not new_rule in path and not missing_found:
+                    if new_rule not in path and not missing_found:
                         path.append(new_rule)
                     return child.predict_proportional(input_data, path,
-                                                      missing_found)
+                                                      missing_found, median,
+                                                      parent=self)
         else:
             # missing value found, the unique path stops
             missing_found = True
+            minimums = []
+            maximums = []
+            population = 0
             for child in self.children:
-                final_distribution = merge_distributions(
-                    final_distribution,
+                (subtree_distribution, subtree_min,
+                 subtree_max, _, subtree_pop, _) = \
                     child.predict_proportional(input_data, path,
-                                               missing_found)[0])
-            return final_distribution, self
+                                               missing_found, median,
+                                               parent=self)
+                if subtree_min is not None:
+                    minimums.append(subtree_min)
+                if subtree_max is not None:
+                    maximums.append(subtree_max)
+                population += subtree_pop
+                final_distribution = merge_distributions(
+                    final_distribution, subtree_distribution)
+            return (final_distribution,
+                    min(minimums) if minimums else None,
+                    max(maximums) if maximums else None, self, population,
+                    self)
 
     def generate_rules(self, depth=0, ids_path=None, subtree=True):
         """Translates a tree model into a set of IF-THEN rules.
@@ -503,6 +538,7 @@ class Tree(object):
             cmv = []
         body = u""
         term_analysis_fields = []
+        item_analysis_fields = []
         children = filter_nodes(self.children, ids=ids_path,
                                 subtree=subtree)
         if children:
@@ -511,8 +547,9 @@ class Tree(object):
                                   none_value(children))
             # the missing is singled out as a special case only when there's
             # no missing branch in the children list
-            if (not has_missing_branch and
-                not self.fields[field]['slug'] in cmv):
+            if not has_missing_branch and \
+                    self.fields[field]["optype"] not in ["text", "items"] and \
+                    self.fields[field]['slug'] not in cmv:
                 body += (u"%sif (%s is None):\n" %
                          (INDENT * depth,
                           map_data(self.fields[field]['slug'], True)))
@@ -531,24 +568,34 @@ class Tree(object):
                 if has_missing_branch and child.predicate.value is not None:
                     negation = u"" if child.predicate.missing else u" not"
                     connection = u"or" if child.predicate.missing else u"and"
-                    pre_condition = (u"%s is%s None %s " % (
-                                     map_data(self.fields[field]['slug'],
-                                              True),
-                                     negation,
-                                     connection))
+                    pre_condition = (
+                        u"%s is%s None %s " % (
+                            map_data(self.fields[field]['slug'], True),
+                            negation,
+                            connection))
                     if not child.predicate.missing:
                         cmv.append(self.fields[field]['slug'])
                 optype = self.fields[field]['optype']
                 if (optype == 'numeric' or optype == 'text' or
-                    child.predicate.value is None):
+                        optype == 'items'
+                        or child.predicate.value is None):
                     value = child.predicate.value
                 else:
                     value = repr(child.predicate.value)
-                if optype == 'text':
+                if optype == 'text' or optype == 'items':
+                    if optype == 'text':
+                        term_analysis_fields.append((field,
+                                                     child.predicate.term))
+                        matching_function = "term_matches"
+                    else:
+                        item_analysis_fields.append((field,
+                                                     child.predicate.term))
+                        matching_function = "item_matches"
+
                     body += (
-                        u"%sif (%sterm_matches(%s, \"%s\", %s\"%s\") %s %s):"
+                        u"%sif (%s%s(%s, \"%s\", %s\"%s\") %s %s):"
                         u"\n" %
-                        (INDENT * depth, pre_condition,
+                        (INDENT * depth, pre_condition, matching_function,
                          map_data(self.fields[field]['slug'],
                                   False),
                          self.fields[field]['slug'],
@@ -557,8 +604,6 @@ class Tree(object):
                          child.predicate.term.replace("\"", "\\\""),
                          PYTHON_OPERATOR[child.predicate.operator],
                          value))
-                    term_analysis_fields.append((field,
-                                                 child.predicate.term))
                 else:
                     operator = (MISSING_OPERATOR[child.predicate.operator] if
                                 child.predicate.value is None else
@@ -578,6 +623,7 @@ class Tree(object):
                                                subtree=subtree)
                 body += next_level[0]
                 term_analysis_fields.extend(next_level[1])
+                item_analysis_fields.extend(next_level[2])
         else:
             if self.fields[self.objective_id]['optype'] == 'numeric':
                 value = self.output
@@ -585,7 +631,7 @@ class Tree(object):
                 value = repr(self.output)
             body = u"%sreturn %s\n" % (INDENT * depth, value)
 
-        return body, term_analysis_fields
+        return body, term_analysis_fields, item_analysis_fields
 
     def python(self, out, docstring, input_map=False,
                ids_path=None, subtree=True):
@@ -615,18 +661,22 @@ class Tree(object):
             (",\n" + " " * depth).join(args))
         predictor_doc = (INDENT + u"\"\"\" " + docstring +
                          u"\n" + INDENT + u"\"\"\"\n")
-        body, term_analysis_predicates = self.python_body(input_map=input_map,
-                                                          ids_path=ids_path,
-                                                          subtree=subtree)
-        terms_body = ""
-        if term_analysis_predicates:
-            terms_body = self.term_analysis_body(term_analysis_predicates)
+        body, term_analysis_predicates, item_analysis_predicates = \
+            self.python_body(input_map=input_map,
+                             ids_path=ids_path,
+                             subtree=subtree)
+        terms_body = u""
+        if term_analysis_predicates or item_analysis_predicates:
+            terms_body = self.term_analysis_body(term_analysis_predicates,
+                                                 item_analysis_predicates)
         predictor += predictor_doc + terms_body + body
         out.write(utf8(predictor))
         out.flush()
 
-    def term_analysis_body(self, term_analysis_predicates):
-        """ Writes auxiliary functions to handle the term analysis fields
+    def term_analysis_body(self, term_analysis_predicates,
+                           item_analysis_predicates):
+        """ Writes auxiliary functions to handle the term and item
+        analysis fields
 
         """
         body = u""
@@ -638,11 +688,15 @@ class Tree(object):
     tm_full_term = '%s'
     tm_all = '%s'
 
-
+"""  % (TM_TOKENS, TM_FULL_TERM, TM_ALL)
+        if term_analysis_predicates:
+            body += """
     def term_matches(text, field_name, term):
         \"\"\" Counts the number of occurences of term and its variants in text
 
         \"\"\"
+        if text is None:
+            text = ""
         forms_list = term_forms[field_name].get(term, [term])
         options = term_analysis[field_name]
         token_mode = options.get('token_mode', tm_tokens)
@@ -687,30 +741,30 @@ class Tree(object):
 
         \"\"\"
         flags = get_tokens_flags(case_sensitive)
-        expression = ur'(\\b|_)%%s(\\b|_)' %% '(\\\\b|_)|(\\\\b|_)'.join(forms_list)
+        expression = ur'(\\b|_)%s(\\b|_)' % '(\\\\b|_)|(\\\\b|_)'.join(forms_list)
         pattern = re.compile(expression, flags=flags)
         matches = re.findall(pattern, text)
         return len(matches)
-""" % (TM_TOKENS, TM_FULL_TERM, TM_ALL)
 
-        term_analysis_options = set(map(lambda x: x[0],
-                                        term_analysis_predicates))
-        term_analysis_predicates = set(term_analysis_predicates)
-        body += """
+"""
+
+            term_analysis_options = set([predicate[0] for predicate in
+                                         term_analysis_predicates])
+            term_analysis_predicates = set(term_analysis_predicates)
+            body += """
     term_analysis = {"""
-        for field_id in term_analysis_options:
-            field = self.fields[field_id]
-            body += """
+            for field_id in term_analysis_options:
+                field = self.fields[field_id]
+                body += """
         \"%s\": {""" % field['slug']
-            for option in field['term_analysis']:
-                if option in TERM_OPTIONS:
-                    body += """
+                for option in field['term_analysis']:
+                    if option in TERM_OPTIONS:
+                        body += """
                 \"%s\": %s,""" % (option, repr(field['term_analysis'][option]))
-            body += """
+                body += """
         },"""
-        body += """
+                body += """
     }"""
-        if term_analysis_predicates:
             term_forms = {}
             fields = self.fields
             for field_id, term in term_analysis_predicates:
@@ -725,17 +779,61 @@ class Tree(object):
                         terms = [term]
                         terms.extend(all_forms.get(term, []))
                         term_forms[field['slug']][term] = terms
-            body += """
+                body += """
     term_forms = {"""
-            for field in term_forms:
-                body += """
-        \"%s\": {""" % field
-                for term in term_forms[field]:
+                for field in term_forms:
                     body += """
+        \"%s\": {""" % field
+                    for term in term_forms[field]:
+                        body += """
             u\"%s\": %s,""" % (term, term_forms[field][term])
-                body += """
+                    body += """
         },
                 """
+                body += """
+    }
+"""
+        if item_analysis_predicates:
+            body += """
+    def item_matches(text, field_name, item):
+        \"\"\" Counts the number of occurences of item in text
+
+        \"\"\"
+        if text is None:
+            text = ""
+        options = item_analysis[field_name]
+        separator = options.get('separator', ' ')
+        regexp = options.get('separator_regexp')
+        if regexp is None:
+            regexp = r\"%s\" % separator
+        return count_items_matches(text, item, regexp)
+
+
+    def count_items_matches(text, item, regexp):
+        \"\"\" Counts the number of occurences of the item in the text
+
+        \"\"\"
+        expression = r'(^|%s)%s($|%s)' % (regexp, item, regexp)
+        pattern = re.compile(expression, flags=re.U)
+        matches = re.findall(pattern, text)
+        return len(matches)
+"""
+
+            item_analysis_options = set([predicate[0] for predicate in
+                                         item_analysis_predicates])
+            item_analysis_predicates = set(item_analysis_predicates)
+            body += """
+    item_analysis = {"""
+            for field_id in item_analysis_options:
+                field = self.fields[field_id]
+                body += """
+        \"%s\": {""" % field['slug']
+                for option in field['item_analysis']:
+                    if option in ITEM_OPTIONS:
+                        body += """
+                \"%s\": %s,""" % (option, repr(field['item_analysis'][option]))
+                body += """
+        },"""
             body += """
     }
 """
@@ -769,7 +867,7 @@ class Tree(object):
             # the missing is singled out as a special case only when there's
             # no missing branch in the children list
             if (not has_missing_branch and
-                    not self.fields[field]['name'] in cmv):
+                    self.fields[field]['name'] not in cmv):
                 conditions.append("ISNULL([%s])" % self.fields[field]['name'])
                 body += (u"%s %s THEN " %
                          (alternate, " AND ".join(conditions)))
@@ -788,28 +886,28 @@ class Tree(object):
                 if has_missing_branch and child.predicate.value is not None:
                     negation = u"" if child.predicate.missing else u"NOT "
                     connection = u"OR" if child.predicate.missing else u"AND"
-                    pre_condition = (u"(%sISNULL([%s]) %s " % (
-                                     negation, self.fields[field]['name'],
-                                     connection))
+                    pre_condition = (
+                        u"(%sISNULL([%s]) %s " % (
+                            negation, self.fields[field]['name'], connection))
                     if not child.predicate.missing:
                         cmv.append(self.fields[field]['name'])
                     post_condition = u")"
                 optype = self.fields[child.predicate.field]['optype']
                 if child.predicate.value is None:
                     value = ""
-                elif optype == 'text':
+                elif optype == 'text' or optype == 'items':
                     return u""
                 elif optype == 'numeric':
                     value = child.predicate.value
                 else:
                     value = repr(child.predicate.value)
 
-                operator = ("" if child.predicate.value is None else
+                operator = (u"" if child.predicate.value is None else
                             PYTHON_OPERATOR[child.predicate.operator])
                 if child.predicate.value is None:
                     pre_condition = (
                         T_MISSING_OPERATOR[child.predicate.operator])
-                    post_condition = ")"
+                    post_condition = u")"
 
                 conditions.append("%s[%s]%s%s%s" % (
                     pre_condition,
@@ -841,3 +939,38 @@ class Tree(object):
         out.write(utf8(body))
         out.flush()
         return True
+
+    def get_nodes_info(self, headers=None, leaves_only=False):
+        """Yields the information associated to each of the tree nodes
+
+        """
+        row = []
+        if not self.regression:
+            category_dict = dict(self.distribution)
+        for header in headers:
+            if header == self.fields[self.objective_id]['name']:
+                row.append(self.output)
+                continue
+            if header in ['confidence', 'error']:
+                row.append(self.confidence)
+                continue
+            if header == 'impurity':
+                row.append(self.impurity)
+                continue
+            if self.regression and header.startswith('bin'):
+                for bin_value, bin_instances in self.distribution:
+                    row.append(bin_value)
+                    row.append(bin_instances)
+                break
+            if not self.regression:
+                row.append(category_dict.get(header))
+        while len(row) < len(headers):
+            row.append(None)
+        if not leaves_only or not self.children:
+            yield row
+
+        if self.children:
+            for child in self.children:
+                for row in child.get_nodes_info(headers,
+                                                leaves_only=leaves_only):
+                    yield row
