@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/env python
 #
-# Copyright 2012-2015 BigML
+# Copyright 2012-2020 BigML
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -18,17 +18,25 @@
 """Miscellaneous utility functions.
 
 """
+
+
 import re
-import unidecode
-from urlparse import urlparse
 import locale
 import sys
 import os
 import json
 import math
 import random
+import ast
+import datetime
+from urlparse import urlparse
+
+import unidecode
 
 
+import bigml.constants as c
+
+PY3 = sys.version > '3'
 DEFAULT_LOCALE = 'en_US.UTF-8'
 WINDOWS_DEFAULT_LOCALE = 'English'
 LOCALE_SYNONYMS = {
@@ -74,13 +82,15 @@ LINKS_REGEX = re.compile((r'''(\[((?:\[[^\]]*\]|[^\[\]])*)\]\([ \t]*()'''
 TYPE_MAP = {
     "categorical": str,
     "numeric": locale.atof,
-    "text": str
+    "text": str,
+    "items": str
 }
 
 PYTHON_TYPE_MAP = {
     "categorical": [unicode, str],
     "numeric": [int, float],
-    "text": [unicode, str]
+    "text": [unicode, str],
+    "items": [unicode, str]
 }
 
 PREDICTIONS_FILE_SUFFIX = '_predictions.csv'
@@ -89,6 +99,13 @@ PROGRESS_BAR_WIDTH = 50
 
 HTTP_INTERNAL_SERVER_ERROR = 500
 
+PRECISION = 5
+
+NUMERIC = "numeric"
+
+DFT_STORAGE = "./storage"
+DFT_STORAGE_FILE = os.path.join(DFT_STORAGE, "BigML_%s.json")
+
 
 def python_map_type(value):
     """Maps a BigML type to equivalent Python types.
@@ -117,8 +134,11 @@ def slugify(name, reserved_keywords=None, prefix=''):
     """
     name = unidecode.unidecode(name).lower()
     name = re.sub(r'\W+', '_', name)
-    if name[0].isdigit():
-        name = "field_" + name
+    try:
+        if name[0].isdigit():
+            name = "field_" + name
+    except IndexError:
+        name = "unnamed_field"
     if reserved_keywords:
         if name in reserved_keywords:
             name = prefix + name
@@ -192,7 +212,7 @@ def sort_fields(fields):
     """
     fathers = [(key, val) for key, val in
                sorted(fields.items(), key=lambda k: k[1]['column_number'])
-               if not 'auto_generated' in val]
+               if 'auto_generated' not in val]
     children = [(key, val) for key, val in
                 sorted(fields.items(), key=lambda k: k[1]['column_number'])
                 if 'auto_generated' in val]
@@ -233,7 +253,7 @@ def locale_synonyms(main_locale, locale_alias):
 
     """
     language_code = main_locale[0:2]
-    if not language_code in LOCALE_SYNONYMS:
+    if language_code not in LOCALE_SYNONYMS:
         return False
     alternatives = LOCALE_SYNONYMS[language_code]
     if isinstance(alternatives[0], basestring):
@@ -254,7 +274,7 @@ def bigml_locale(locale_alias):
        the locale user code has been correctly mapped. None otherwise.
     """
     language_code = locale_alias.lower()[0:2]
-    if not language_code in LOCALE_SYNONYMS:
+    if language_code not in LOCALE_SYNONYMS:
         return None
     alternatives = LOCALE_SYNONYMS[language_code]
     if isinstance(alternatives[0], basestring):
@@ -291,7 +311,7 @@ def find_locale(data_locale=DEFAULT_LOCALE, verbose=False):
                         break
                     except locale.Error:
                         pass
-                if not new_locale is None:
+                if new_locale is not None:
                     break
             else:
                 try:
@@ -347,16 +367,25 @@ def reset_console_line(out=sys.stdout, length=PROGRESS_BAR_WIDTH):
     out.flush()
 
 
-def console_log(message, out=sys.stdout, length=PROGRESS_BAR_WIDTH):
+def console_log(message, out=sys.stdout, length=PROGRESS_BAR_WIDTH,
+                reset=False):
     """Prints the message to the given output
 
+       :param out: output handler
+       :param length: maximum length
+       :param reset: whether the line has to be reused and cursor reset to
+              the beggining of it
     """
-    clear_console_line(out=out, length=length)
-    reset_console_line(out=out, length=length)
-    if out==sys.stdout and sys.platform == "win32" and sys.stdout.isatty():
-        message = message.decode('utf8').encode('850')
+    if reset:
+        clear_console_line(out=out, length=length)
+        reset_console_line(out=out, length=length)
+    if (out == sys.stdout and sys.platform == "win32" and sys.stdout.isatty()
+            and not PY3):
+        os_encoding = sys.stdout.encoding
+        message = message.decode('utf8').encode(os_encoding, 'replace')
     out.write(message)
-    reset_console_line(out=out, length=length)
+    if reset:
+        reset_console_line(out=out, length=length)
 
 
 def get_csv_delimiter():
@@ -385,7 +414,27 @@ def cast(input_data, fields):
 
     """
     for (key, value) in input_data.items():
-        if (
+        # strings given as booleans
+        if isinstance(value, bool) and \
+                fields[key]['optype'] == 'categorical' and \
+                len(fields[key]['summary']['categories']) == 2:
+            try:
+                booleans = {}
+                categories = [category for category, _ in \
+                    fields[key]['summary']['categories']]
+                # checking which string represents the boolean
+                for category in categories:
+                    bool_key = 'True' if ast.literal_eval( \
+                        category.capitalize()) else 'False'
+                    booleans[bool_key] = category
+                # converting boolean to the corresponding string
+                input_data.update({key: booleans[str(value)]})
+            except ValueError:
+                raise ValueError(u"Mismatch input data type in field "
+                                 u"\"%s\" for value %s. String expected" %
+                                 (fields[key]['name'], value))
+        # numerics given as strings
+        elif (
                 (fields[key]['optype'] == 'numeric' and
                  isinstance(value, basestring)) or
                 (fields[key]['optype'] != 'numeric' and
@@ -401,6 +450,11 @@ def cast(input_data, fields):
                                  u"\"%s\" for value %s." %
                                  (fields[key]['name'],
                                   value))
+        elif (fields[key]['optype'] == 'numeric' and
+              isinstance(value, bool)):
+            raise ValueError(u"Mismatch input data type in field "
+                             u"\"%s\" for value %s. Numeric expected." %
+                             (fields[key]['name'], value))
 
 
 def check_dir(path):
@@ -442,31 +496,80 @@ def empty_resource():
                 "message": "The resource couldn't be created"}})
 
 
+def get_status(resource):
+    """Extracts status info if present or sets the default if public
+
+    """
+    if not isinstance(resource, dict):
+        raise ValueError("We need a complete resource to extract its status")
+    if 'object' in resource:
+        if resource['object'] is None:
+            raise ValueError("The resource has no status info\n%s" % resource)
+        resource = resource['object']
+    if not resource.get('private', True) or resource.get('status') is None:
+        status = {'code': c.FINISHED}
+    else:
+        status = resource['status']
+    return status
+
+
 def maybe_save(resource_id, path,
                code=None, location=None,
                resource=None, error=None):
-    """ Builds the resource dict response and saves it if a path is provided.
+    """Builds the resource dict response and saves it if a path is provided.
 
-        The resource is saved in a local repo json file in the given path.
+    The resource is saved in a local repo json file in the given path.
+    Only final resources are stored. Final resources should be FINISHED or
+    FAILED
 
     """
     resource = resource_structure(code, resource_id, location, resource, error)
-
-    if path is not None and resource_id is not None:
-        try:
-            resource_json = json.dumps(resource)
-        except ValueError:
-            print "The resource has an invalid JSON format"
-        try:
-            resource_file_name = "%s%s%s" % (path, os.sep,
-                                             resource_id.replace('/', '_'))
-            with open(resource_file_name, "w", 0) as resource_file:
-                resource_file.write(resource_json)
-        except IOError:
-            print "Failed writing resource to %s" % resource_file_name
+    if resource_id is not None and path is not None and \
+            is_status_final(resource):
+        resource_file_name = "%s%s%s" % (path, os.sep,
+                                         resource_id.replace('/', '_'))
+        save_json(resource, resource_file_name)
     return resource
 
 
+def is_status_final(resource):
+    """Try whether a resource is in a final state
+
+    """
+    try:
+        status = get_status(resource)
+    except ValueError:
+        status['code'] = None
+    return status['code'] in [c.FINISHED, c.FAULTY]
+
+
+def save_json(resource, path):
+    """Stores the resource in the user-given path in a JSON format
+
+    """
+    try:
+        resource_json = json.dumps(resource)
+        return save(resource_json, path)
+    except ValueError:
+        print "The resource has an invalid JSON format"
+    except IOError:
+        print "Failed writing resource to %s" % path
+
+
+def save(content, path):
+    """Stores content in an utf-8 file
+
+    """
+    if path is None:
+        datestamp = datetime.datetime.now().strftime("%a%b%d%y_%H%M%S")
+        path = DFT_STORAGE_FILE % datestamp
+    check_dir(os.path.dirname(path))
+    with open(path, "wb", 0) as file_handler:
+        content = content.encode('UTF-8')
+        file_handler.write(content)
+    return path
+
+
 def plural(text, num):
     """Pluralizer: adds "s" at the end of a string if a given number is > 1
 
@@ -476,12 +579,55 @@ def plural(text, num):
 
 def get_exponential_wait(wait_time, retry_count):
     """Computes the exponential wait time used in next request using the
-       base values provided by the user:
-        - wait_time: starting wait time
-        - retries: total number of retries
-        - retries_left: retries left
+    base values provided by the user:
+        - wait_time: starting wait time (seconds)
+        - retries_count: number of retries
 
     """
     delta = (retry_count ** 2) * wait_time / 2
     exp_factor = delta if retry_count > 1 else 0
     return wait_time + math.floor(random.random() * exp_factor)
+
+
+def check_no_missing_numerics(input_data, fields, weight_field=None):
+    """Checks whether some numeric fields are missing in the input data
+
+    """
+    for field_id, field in fields.items():
+        if (field['optype'] == NUMERIC and (weight_field is None or \
+                field_id != weight_field) and \
+                not field_id in input_data):
+            raise ValueError("Failed to predict. Input"
+                             " data must contain values for all numeric"
+                             " fields to get a prediction.")
+
+def check_no_training_missings(input_data, fields, weight_field=None,
+                               objective_id=None):
+    """Checks whether some input fields are missing in the input data
+    while not training data has no missings in that field
+
+    """
+    for field_id, field in fields.items():
+        if (field["optype"] != "datetime" and \
+                field_id not in input_data and \
+                field['summary']['missing_count'] == 0 and \
+                (weight_field is None or field_id != weight_field) and \
+                (objective_id is None or field_id != objective_id)):
+            raise ValueError("Failed to predict. Input"
+                             " data must contain values for field '%s' "
+                             "to get a prediction." % field['name'])
+
+
+def flatten(inner_array):
+    """ Flattens an array with inner arrays
+
+    """
+    new_array = []
+
+    for element in inner_array:
+        if isinstance(element, list):
+            new_array.extend(element)
+        else:
+            new_array.append(element)
+
+    return new_array
